var sidebarController = function(
  $scope,
  $timeout,
  $filter,
  $q,
  ChatAPI,
  ChatUser,
  ChatSidebar,
  ChatPopups,
  ChatContacts,
  ChatConversations,
  ChatStatus,
  ChatProductTags,
  ChatErrors,
  ChatNavigator,
  ChatMessageLog,
  ChatToaster,
  ChatUtils
) {
  var vm = this;

  // Display messages (errors, infos, warnings, etc)
  vm.toaster = new ChatToaster();

  // Errors =========================
  var Errors = ChatSidebar.errors();
  vm.errors = Errors.errors;
  vm.getError = Errors.getError;
  Errors.setError(1, ChatMessageLog.ERROR.LOGIN, "icon-img icon-disconnected");

  // Toggle Chat ========================
  vm.isCollapsed = ChatSidebar.settings.collapsed;
  vm.chatToggle = function() {
    ChatSidebar.toggle();
  };

  vm.chatOpen = function() {
    ChatSidebar.open();
  };

  window.inGaiaChat_API.on("chatVisibilityChange", function(state) {
    $timeout(function() {
      vm.isCollapsed = state.collapsed;
    }, 0);
  });

  window.inGaiaChat_API.on("chatStart", function(connectionData) {
    if (!connectionData) {
      window.inGaiaChat_API.emit("chatConnectionFail");
      return false;
    }

    ChatUser.set(connectionData.user);
    vm.User = connectionData.user;
    vm.reportsUrl = null;

    if (connectionData.reportsUrl) {
      vm.reportsUrl = connectionData.reportsUrl;
    }

    vm.config = connectionData.config;
    vm.loadedSettings = false;
    vm.connecting = true;

    // Settings
    if (vm.config) {
      vm.settings = ChatSidebar.mergeSettings(vm.config);
    } else {
      vm.settings = ChatSidebar.settings;
    }

    // Start connection with Chat API
    ChatAPI.start(connectionData)
      .then(
        function() {
          window.inGaiaChat_API.emit("chatConnected");
          Errors.removeError(1);

          var saveSettings = function(data) {
            var userParams = {
              id: vm.User.id,
              productTag: vm.User.productTag
            };

            var onErrorSaving = function(err) {
              vm.toaster.add({
                icon: "ing-gear",
                type: "error",
                message: ChatMessageLog.SETTINGS.ERROR.SAVE,
                dismissable: true
              });

              console.log(ChatMessageLog.SETTINGS.ERROR.SAVE);
            };

            ChatAPI.saveSettings(
              userParams.id,
              userParams.productTag,
              data
            ).then(null, onErrorSaving);
          };

          // SETTINGS =====================================
          var loadSettings = function() {
            // Display loading preferences
            var toast = {
              icon: "ing-gear",
              type: "info",
              message: ChatMessageLog.SETTINGS.LOADING,
              dismissable: true
            };

            vm.toaster.add(toast);

            // Get user settings
            ChatAPI.loadSettings({
              userId: vm.User.id,
              productTag: vm.User.productTag
            })
              .then(
                function(response) {
                  vm.toaster.remove(toast);

                  if (response.data.collapsed != vm.settings.collapsed) {
                    vm.settings.collapsed = response.data.collapsed;
                    window.inGaiaChat_API.emit("chatVisibilityChange", {
                      collapsed: response.data.collapsed
                    });
                  }

                  if (response.data.notification)
                    ChatNavigator.notification.askPermission();

                  vm.settings = ChatSidebar.mergeSettings(response.data);

                  window.inGaiaChat_API.emit("chatSettingsLoaded", vm.settings);

                  // Start Watching for changes in settings
                  $scope.$watchCollection(
                    "vm.settings",
                    function(newVal, oldVal) {
                      if (newVal != oldVal) {
                        if (
                          newVal.notification &&
                          newVal.notification != oldVal.notification
                        )
                          ChatNavigator.notification.askPermission();
                        if (newVal.layoutMin != oldVal.layoutMin) {
                          window.inGaiaChat_API.emit(
                            "chatWidgetChange",
                            newVal.layoutMin
                          );
                        }
                        saveSettings(newVal);
                      }
                    },
                    true
                  );

                  // Notify settings watcher to save new settings
                  window.inGaiaChat_API.on("chatStatusChange", function(
                    statusId
                  ) {
                    vm.settings.lastStatus = statusId;
                    $timeout(function() {
                      // Display error message if user change status to offline
                      if (statusId == ChatStatus.enums.offline)
                        Errors.setError(
                          4,
                          ChatMessageLog.WARNING.CHANGE_STATUS_OFFLINE,
                          "icon-img icon-change-status"
                        );
                      else Errors.removeError(4);
                    }, 0);
                  });
                },
                function(err) {
                  vm.toaster.add({
                    icon: "ing-gear",
                    type: "error",
                    message: ChatMessageLog.SETTINGS.ERROR.LOAD,
                    dismissable: true
                  });
                }
              )
              .finally(function() {
                // Initialize current status ==========================================
                vm.status = ChatStatus.init(vm.settings.lastStatus);
                vm.loadedSettings = true;
              });
          };
          loadSettings();
          // END SETTINGS =====================================

          // SEARCH ==========================================
          vm.search = ChatSidebar.search;
          vm.search.clear = function() {
            $timeout(function() {
              vm.searchContactsResult = [];
              vm.search.text = "";
              vm.search.close();
            }, 0);
          };

          // TABS ===============================================
          vm.tab = ChatSidebar.tab.init();

          // FILTERS ============================================
          vm.companyFilters = [];
          vm.filterSelected = null;

          // Append user filters array
          if (vm.User.companyFilters) {
            vm.companyFilters = vm.User.companyFilters;
            vm.filterSelected = vm.companyFilters[0];
          }

          $scope.$watch("vm.filterSelected", function(newVal) {
            if (newVal) {
              vm.search.clear();
              getContacts();
            }
          });
          // End FILTERS =========================================

          vm.conversations = ChatConversations;
          vm.contacts = ChatContacts;
          // POPUPS
          vm.popups = ChatPopups;
          vm.popups.grouped = [];
          vm.openPopup = function(contact) {
            vm.popups.openPopup(contact);
          };

          /**
           * Create group
           * @method
           */
          vm.createGroup = function(groupName) {
            var isGroup, notifyOtherConnections, name;
            isGroup = true;
            notifyOtherConnections = true;
            name = groupName && groupName.length ? groupName : null;

            // Add logged user to group
            vm.addParticipantList.splice(0, 0, vm.User);

            ChatAPI.proxy()
              .server.firstContact(
                vm.addParticipantList,
                null,
                isGroup,
                notifyOtherConnections,
                name
              )
              .then(
                function(group) {
                  vm.addParticipantList = [];
                  vm.tab.open("conversations");
                  vm.newGroupName = "";
                },
                function(err) {
                  $timeout(function() {
                    vm.addParticipantList = [];
                  }, 0);
                }
              );
          };

          vm.searchAutoCompleteContacts = ChatContacts.search;

          // CONVERSATIONS ====================================
          /**
           * Return an Array of conversations passing a limit and offset (pagination).
           * @method
           * @return {Array}
           */
          var getConversations = function(callback, options) {
            vm.search.clear();
            options = options || null;
            var onErrorConversations = function(err) {
              $timeout(function() {
                Errors.setError(
                  2,
                  ChatMessageLog.CONVERSATIONS.ERROR.LOAD,
                  "icon-img icon-user-chat",
                  "error",
                  true
                );
              }, 0);
            };
            vm.conversations.getConversations(
              options,
              callback,
              onErrorConversations
            );
          };

          getConversations(function(response) {
            vm.conversations.list = response;
            ChatPopups.history().open();
            // Open contacts tab if there's no conversation yet
            if (!response.length) vm.tab.open("contacts");
          });

          /**
           * Fetch conversations list
           * @method
           * @return {Array}
           */
          vm.getMoreConversations = function() {
            if (
              ChatConversations.count > ChatConversations.list.length &&
              !ChatConversations.pending
            ) {
              var options = {
                id: vm.User.id,
                offset:
                  vm.conversations.defaults.limit +
                  vm.conversations.defaults.offset
              };
              var appendConversations = function(response) {
                response.forEach(function(c) {
                  vm.conversations.list.push(c);
                });
              };
              getConversations(appendConversations, options);
            }
          };

          // Search for conversations
          // vm.searchConversations = function (name, filterCount) {
          //     if (vm.conversations.count > 0) {
          //         if (name == "")
          //             vm.searchConversationsResult = [];
          //
          //         if (filterCount == 0) {
          //             if (name && name.length >= 2) {
          //                 // Search for contacts inside current list, if is not found, request to server
          //                 var params = {
          //                     id: vm.User.id,
          //                     name: name,
          //                     limit: 5,
          //                     offset: 0
          //                 };
          //
          //                 vm.conversations.pending = true;
          //
          //                 ChatAPI.searchConversation(params).then(function (response) {
          //                     vm.searchConversationsResult = response.data.data;
          //                 }, function (err) {
          //                     Errors.setError(3, ChatMessageLog.CONVERSATIONS.ERROR.SEARCH, 'icon-img icon-not-found');
          //                     console.log(err.statusText);
          //                 }).finally(function () {
          //                     vm.conversations.pending = false;
          //                 });
          //             }
          //         }
          //     }
          // };

          // CONTACTS ====================================
          /**
           * Return an Array of contacts.
           * @method
           * @return {Array}
           */
          var getContacts = function(callback, options) {
            var onErrorContacts = function(err) {
              Errors.setError(
                5,
                ChatMessageLog.CONTACTS.ERROR.LOAD,
                "icon-img icon-not-found"
              );
            };

            var onSuccessContacts = function(response) {
              if (callback) callback(response);
              else vm.contacts.list = response;

              if (response.length == 0 && !callback) {
                if (vm.filterSelected.id == ChatProductTags.communitygaia) {
                  Errors.setError(
                    5,
                    ChatMessageLog.CONTACTS.SEARCH.NO_RESULTS.COMMUNITY,
                    "icon-img icon-not-found"
                  );
                } else if (vm.filterSelected.id == ChatProductTags.incorp) {
                  Errors.setError(
                    5,
                    ChatMessageLog.CONTACTS.SEARCH.NO_RESULTS.INCORP,
                    "icon-img icon-not-found"
                  );
                } else {
                  Errors.setError(
                    5,
                    ChatMessageLog.CONTACTS.SEARCH.NO_RESULTS.DEFAULT,
                    "icon-img icon-not-found"
                  );
                }
              } else {
                Errors.removeError(5);
                $timeout(function() {
                  ChatContacts.mergeBlockedWithContacts();
                }, 100);
              }
            };

            switch (vm.filterSelected.id) {
              case ChatProductTags.realestatenework:
                var params = {
                  networkId: vm.filterSelected.id
                };
                if (options != undefined) {
                  if (options.limit) params.limit = options.limit;
                  if (options.offset) params.offset = options.offset;
                }
                ChatContacts.getContactsFromNetwork(
                  vm.filterSelected.url,
                  params,
                  onSuccessContacts,
                  onErrorContacts
                );
                break;
              default:
                ChatContacts.getContacts(
                  vm.filterSelected.url,
                  options,
                  onSuccessContacts,
                  onErrorContacts
                );
            }
          };

          /**
           * Return an Array of contacts passing a limit and offset (pagination).
           * @method
           * @return {Array}
           */
          vm.getMoreContacts = function() {
            vm.search.clear();

            // Don't do anything if search results is not empty
            if (vm.searchContactsResult && vm.searchContactsResult.length > 0) {
              return false;
            }

            if (!ChatContacts.pending) {
              // Dont't request if is from incorp or communitygaia
              if (
                vm.filterSelected.id != ChatProductTags.incorp &&
                vm.filterSelected.id != ChatProductTags.communitygaia
              ) {
                var options = {
                  offset: vm.contacts.list.length
                };
                var appendContacts = function(response) {
                  response.forEach(function(c) {
                    vm.contacts.list.push(c);
                  });
                };
                getContacts(appendContacts, options);
              }
            }
          };

          /**
           * Search for conversations
           * @method
           * @return {Array}
           */
          var searchContacts = function(name) {
            // Incorp is not included in full search, so we just filter all results from first request
            if (vm.filterSelected.id == ChatProductTags.incorp) return false;

            if (name != undefined && name != "") {
              ChatContacts.search(name).then(
                function(contacts) {
                  if (contacts.length == 0) {
                    vm.searchContactsResult = contacts;
                    Errors.setError(
                      5,
                      ChatMessageLog.CONTACTS.SEARCH.NO_RESULTS.DEFAULT,
                      "icon-img icon-not-found",
                      "error",
                      true
                    );
                  } else {
                    vm.searchContactsResult = mergeBlockedWithContacts(
                      contacts
                    );
                    Errors.removeError(5);
                  }
                },
                function(err) {
                  Errors.setError(
                    5,
                    ChatMessageLog.CONTACTS.ERROR.SEARCH,
                    "icon-img icon-not-found",
                    "error",
                    true
                  );
                }
              );
            } else {
              vm.searchContactsResult = [];
              Errors.removeError(5);
            }
          };

          vm.searchContacts = ChatUtils.debounce(searchContacts, 300);

          /**
           * Return an Array of contacts merged with blocked list.
           * @method
           * @return {Array}
           */
          var mergeBlockedWithContacts = function(list) {
            ChatContacts.blocked.forEach(function(blocked) {
              list.forEach(function(contact) {
                if (blocked.id == contact.id) {
                  contact.isBlocked = true;
                }
              });
            });
            return list;
          };

          /**
           * Return an Array of blocked contacts.
           * @method
           * @return {Array}
           */
          var getBlockedList = function(callback) {
            var userParams = {
              id: vm.User.id,
              productTag: vm.User.productTag
            };

            ChatAPI.blockedList(userParams).then(
              function(response) {
                vm.blockedList = ChatContacts.blocked = response.data;

                // Set all search result as not blocked
                ChatContacts.list.forEach(function(contact) {
                  contact.isBlocked = false;
                });

                // Then merge new result of blocked contacts
                ChatContacts.mergeBlockedWithContacts();

                if (callback) {
                  callback(response.data);
                }
              },
              function(err) {
                vm.toaster.add({
                  type: "error",
                  message: ChatMessageLog.CONTACTS.ERROR.LOAD_BLOCKED,
                  dismissable: true
                });

                console.log(ChatMessageLog.CONTACTS.ERROR.LOAD_BLOCKED);
              }
            );
          };

          getBlockedList();

          /**
           * Append new message from contact to conversation
           * @method
           */
          var onReceiveMessage = function(data) {
            window.inGaiaChat_API.emit("chatReceiveMessage", data);
            var room = getConversationByRoomId(data.roomId);
            if (room) {
              $timeout(function() {
                removeTyping(room.roomId);
                // Increment messages unread
                if (data.message.sender.id != vm.User.id) {
                  if (!data.message.system) {
                    room.unreadMessages++;
                    window.inGaiaChat_API.emit("chatUpdateRoomUnreads", room);
                    ChatConversations.unreads++;
                    window.inGaiaChat_API.emit(
                      "chatNotificationUpdate",
                      ChatConversations.unreads
                    );
                    // Show navigator push notification
                    if (
                      vm.settings.notification &&
                      vm.settings.status != ChatStatus.enums.offline
                    ) {
                      var sender = "";
                      if (room.isGroup) {
                        sender = room;
                        sender.photo = sender.photo
                          ? sender.photo
                          : "https://chat.ingaia.com.br/assets/img/icon-group.png";
                      } else {
                        sender = room.participant[0];
                        sender.photo = sender.photo
                          ? sender.photo
                          : "https://chat.ingaia.com.br/assets/img/icon-single-user.png";
                      }
                      ChatNavigator.notification.notify(
                        sender,
                        $filter("colonToCode")(data.message.message)
                      );
                    }
                    // Play sound if is enable and message came from contact
                    if (vm.settings.sound) ChatNavigator.audio.play();
                  }
                }
              }, 0);

              // Replace last message received/sent
              room.lastMessage = data.message.message;
              room.lastMessageDate = data.message.messageDate;
              room.userTyping = null;
            }
          };

          // Notify user when contact is typing from the other side
          var inactiveTypingTimeout, clearTypingTimeout;

          /**
           * Add typing flag to conversation
           * @method
           */
          var onReceiveTyping = function(data) {
            var room = getConversationByRoomId(data.roomId);

            if (room) {
              if (
                room.roomId == data.roomId &&
                data.participant.id != vm.User.id
              ) {
                var popupOpened = null;
                var typingData = {
                  id: data.participant.id,
                  name: data.participant.name,
                  photo: data.participant.photo,
                  from:
                    data.participant.name + " " + ChatMessageLog.USER.IS_TYPING,
                  active: true
                };

                if (vm.popups.list.length) {
                  vm.popups.list.forEach(function(popup) {
                    if (
                      popup.roomId == data.roomId &&
                      data.participant.id != vm.User.id
                    ) {
                      popupOpened = popup;
                    }
                  });
                  if (popupOpened) popupOpened.userTyping = typingData;
                }

                room.userTyping = typingData;

                // Cancel previous inactiveTypingTimeout
                if (inactiveTypingTimeout != undefined)
                  $timeout.cancel(inactiveTypingTimeout);

                inactiveTypingTimeout = $timeout(function() {
                  if (room.userTyping) room.userTyping.active = false;
                  if (popupOpened && popupOpened.userTyping)
                    popupOpened.userTyping.active = false;
                }, 10000);

                // Cancel previous clearTypingTimeout
                if (clearTypingTimeout != undefined)
                  $timeout.cancel(clearTypingTimeout);

                clearTypingTimeout = $timeout(function() {
                  room.userTyping = null;
                  if (popupOpened && popupOpened.userTyping)
                    popupOpened.userTyping = null;
                }, 15000);
              }
            }
          };

          /**
           * Remove typing flag from conversation
           * @method
           */
          var removeTyping = function(roomId) {
            var room = getConversationByRoomId(roomId);
            if (room && room.userTyping) room.userTyping = null;
            if (vm.popups.list.length) {
              vm.popups.list.forEach(function(popup) {
                if (popup.roomId == roomId && popup.userTyping) {
                  $timeout(function() {
                    popup.userTyping = null;
                  }, 0);
                }
              });
            }
            if (inactiveTypingTimeout != undefined)
              $timeout.cancel(inactiveTypingTimeout);
            if (clearTypingTimeout != undefined)
              $timeout.cancel(clearTypingTimeout);
          };

          /**
           * Find contact in list and replace current status from response data.
           * @method
           */
          var updateStatus = function(list, data) {
            list.map(function(item) {
              if (item.participant) {
                // Conversation list
                if (item.participant.length) {
                  if (item.participant[0].id == data.id) {
                    $timeout(function() {
                      item.participant[0].status = data.status;
                    }, 0);
                  }
                }
              } else {
                // Contact list
                if (item.id == data.id) {
                  $timeout(function() {
                    item.status = data.status;
                  }, 0);
                }
              }
            });
          };

          /**
           * Watch for status change
           * @method
           */
          var onUserChangeStatus = function(data) {
            window.inGaiaChat_API.emit("chatUserChangeStatus", data);
            // Update status from user inside conversation list
            updateStatus(vm.conversations.list, data);
            // Update status from user inside contact list
            updateStatus(vm.contacts.list, data);
          };

          /**
           * Push new contact to the list
           * @method
           */
          var updateContactList = function(conversation) {
            var room = getConversationByRoomId(conversation.roomId);
            if (room) {
              return false;
            } else {
              // Remove logged user from participant array
              var participants = conversation.participant.filter(function(c) {
                return c.id != vm.User.id;
              });
              conversation.participant = participants;

              ChatConversations.unreads += conversation.unreadMessages;
              window.inGaiaChat_API.emit(
                "chatNotificationUpdate",
                ChatConversations.unreads
              );

              $timeout(function() {
                vm.conversations.list.splice(0, 0, conversation);
              }, 0);

              if (
                vm.settings.status != ChatStatus.enums.offline &&
                !ChatPopups.isOpen(conversation, false)
              ) {
                var collapsed =
                  conversation.admin.id == vm.User.id ? false : true;
                ChatPopups.openPopup(conversation, collapsed);
              }
            }
          };

          /**
           * Reconnect chat.
           * @method
           */
          var Reconnect = {
            currentAttempt: 1,
            maxAttempts: 6,
            delay: 5000,
            reconnecting: false,
            clear: function() {
              this.currentAttempt = 1;
              this.reconnecting = false;
            },
            start: function(error) {
              var self = this;

              if (!self.reconnecting) {
                self.reconnecting = true;

                if (error != undefined) {
                  // If error is defined, then the disconnection was caused by server error,
                  // We need to set the hub state do disconnected because the state is not changed automatically
                  $.connection.hub.state =
                    $.signalR.connectionState.disconnected;
                }

                var toast = {
                  icon: "ing-exclamation-circle",
                  type: "warn",
                  message: ChatMessageLog.ACTIONS.RECONNECTING
                };

                vm.toaster.add(toast);

                var connectInterval = setInterval(function() {
                  // Errors.setError(6, 'Reconectando...', 'ing-exclamation-circle', 'warn');

                  // if (self.currentAttempt >= self.maxAttempts) {
                  //     clearInterval(connectInterval);
                  //     self.clear();
                  //     Errors.setError(1, ChatMessageLog.ERROR.RECONNECTION, 'icon-img icon-disconnected');
                  //     return;
                  // }

                  var state = $.connection.hub.state;
                  var connectionState = $.signalR.connectionState;

                  if (
                    state == connectionState.disconnected ||
                    state == connectionState.connecting
                  ) {
                    ChatAPI.connectHub().fail(function(err) {
                      vm.toaster.remove(toast);
                      vm.toaster.add({
                        icon: "ing-exclamation-circle",
                        type: "error",
                        message: ChatMessageLog.ERROR.RECONNECTION
                      });
                      console.error(err);
                    });
                    self.currentAttempt++;
                  } else if (state == connectionState.connected) {
                    clearInterval(connectInterval);

                    ChatAPI.connectApi(connectionData.user).then(
                      function() {
                        vm.toaster.remove(toast);
                        self.clear();
                        window.inGaiaChat_API.emit("chatReconnected");
                      },
                      function(err) {
                        vm.toaster.add({
                          icon: "ing-exclamation-circle",
                          type: "error",
                          message: ChatMessageLog.ERROR.RECONNECTION
                        });
                      }
                    );
                  }
                }, self.delay);
              }
            }
          };
          vm.reconnect = Reconnect;
          // END RECONNECT =======================================

          /**
           * Display error message when disconnected and invoke reconnect method when chat is disconnected
           * @method
           */
          var onDisconnected = function(error) {
            window.inGaiaChat_API.open(); // Open sidebar in case is collapsed
            Reconnect.start(error);
          };

          /**
           * Get a conversation by room id
           * @method
           * return {Object}
           */
          var getConversationByRoomId = function(roomId) {
            return ChatConversations.list.filter(function(conversation) {
              return conversation.roomId == roomId;
            })[0];
          };

          /**
           * Remove conversation if user was removed and close popup (if it's open)
           * @method
           */
          var removeConversationByRoomId = function(roomId, selfRemoved) {
            var room = getConversationByRoomId(roomId);
            if (room && (room.isGroup || !!selfRemoved)) {
              ChatPopups.closePopup(room);

              var list = ChatConversations.list.filter(function(conversation) {
                return conversation.roomId != roomId;
              });

              ChatConversations.list = list;

              ChatConversations.unreads -= room.unreadMessages;
              window.inGaiaChat_API.emit(
                "chatNotificationUpdate",
                ChatConversations.unreads
              );
            }
          };

          /**
           * Remove a participant from a conversation
           * @method
           */
          var removeParticipants = function(roomId, participants, selfRemoved) {
            var room = getConversationByRoomId(roomId);

            if (room) {
              participants.map(function(participant) {
                // If logged user is the removed participant
                if (participant.id == vm.User.id) {
                  removeConversationByRoomId(roomId);
                } else {
                  var withoutRemoved = room.participant.filter(function(p) {
                    return p.id != participant.id;
                  });
                  room.participant = withoutRemoved;
                }
              });
            }
          };

          /**
           * Add a participant to a conversation
           * @method
           */
          var onAddParticipant = function(roomId, participants) {
            var room = getConversationByRoomId(roomId);
            if (room) {
              participants.forEach(function(participant) {
                room.participant.push(participant);
              });
            }
          };

          /**
           * Update the name of a conversation (Room)
           * @method
           */
          var onRoomEdited = function(data) {
            var room = getConversationByRoomId(data.roomId);
            if (room) {
              room.name = data.name;
            }
            window.inGaiaChat_API.emit("chatRoomEdited", data);
          };

          /**
           * Open a popup to confirm a partnership between incorp or community
           * @method
           */
          var onFirstContactCustom = function(data) {
            $timeout(function() {
              ChatPopups.openPopup(data.user, false, data.property);
            }, 0);
          };

          /**
           * Emit an event to notify a popup when the messages were read
           * @method
           */
          var onMessageAsRead = function(roomId) {
            var room = getConversationByRoomId(roomId);
            if (room && room.unreadMessages > 0) {
              ChatConversations.unreads -= room.unreadMessages;
              room.unreadMessages = 0;
              window.inGaiaChat_API.emit(
                "chatNotificationUpdate",
                ChatConversations.unreads
              );
              window.inGaiaChat_API.emit("chatMessageAsRead", roomId);
            }
          };

          // APP EVENT LISTENERS =======================================
          window.inGaiaChat_API.on("updateBlockedList", getBlockedList);
          window.inGaiaChat_API.on("updateConversationsList", getConversations);
          window.inGaiaChat_API.on("chatDisconnected", onDisconnected);
          window.inGaiaChat_API.on("firstContactCustom", onFirstContactCustom);

          // API EVENT LISTENERS =======================================
          ChatAPI.proxy().on("receiveMessage", onReceiveMessage);
          ChatAPI.proxy().on("receiveTyping", onReceiveTyping);
          ChatAPI.proxy().on("userStatus", onUserChangeStatus);
          ChatAPI.proxy().on("updateContactList", updateContactList);
          ChatAPI.proxy().on("onRoomExcluded", removeConversationByRoomId);
          ChatAPI.proxy().on("onRemovedParticipant", removeParticipants);
          ChatAPI.proxy().on("onAddParticipant", onAddParticipant);
          ChatAPI.proxy().on("onRoomEdited", onRoomEdited);
          ChatAPI.proxy().on("onMessageAsRead", onMessageAsRead);
        },
        function(err) {
          window.inGaiaChat_API.emit("chatVisibilityChange", {
            collapsed: ChatSidebar.settings.collapsed
          });
          Errors.setError(
            1,
            ChatMessageLog.ERROR.CONNECTION,
            "icon-img icon-disconnected"
          );
        }
      )
      .finally(function() {
        vm.connecting = false;
      });
  });
};

sidebarController.$inject = [
  "$scope",
  "$timeout",
  "$filter",
  "$q",
  "ChatAPI",
  "ChatUser",
  "ChatSidebar",
  "ChatPopups",
  "ChatContacts",
  "ChatConversations",
  "ChatStatus",
  "ChatProductTags",
  "ChatErrors",
  "ChatNavigator",
  "ChatMessageLog",
  "ChatToaster",
  "ChatUtils"
];

var sidebarLink = function(scope, element, attrs) {
  // Reposition sidebar component when window is loaded
  $(window).load(function() {
    window.inGaiaChat_API.emit("chatUpdateWidgetPosition");
  });
};

var componentsModules = require("../components.module");

componentsModules.directive("ingaiachat", function() {
  return {
    restrict: "EA",
    templateUrl: "ingaia-chat/login/login.tpl.html",
    replace: true,
    scope: {},
    controllerAs: "vm",
    controller: sidebarController,
    link: sidebarLink
  };
});
