var componentsModules = require('../components.module');
componentsModules.directive("chatPopover", [
    '$compile', '$templateCache', '$window', '$timeout', 'alignSvc',
    function ($compile, $templateCache, $window, $timeout, alignSvc) {
        return {
            scope: false,
            link: function (scope, element, attrs) {
                var popover, align, trigger, delay, tplCache, pos, enable;
                align = attrs.chatPopoverAlign || 'top';
                trigger = attrs.chatPopoverTrigger || 'hover';
                delay = parseInt(attrs.chatPopoverDelay) || 750;
                enable = scope.$eval(attrs.chatPopoverEnable);
                if (enable === undefined) {
                    enable = true;
                }
                tplCache = $templateCache.get(attrs.chatPopover);

                // Show popover
                function show() {
                    if (enable) {
                        var template = angular.element(document.createElement('div'));
                        template = template.append('<div class="arrow"></div>');
                        template = template.append(tplCache);
                        template.addClass('chat-popover fade ' + align);

                        var newScope = scope.$new(true);
                        newScope.data = scope.$eval(attrs.chatPopoverData);
                        popover = $compile(template)(newScope);

                        // Append to body
                        angular.element('body').after(popover);

                        $timeout(function () {
                            if (popover) {
                                popover.show();
                            }
                        }, delay).then(function () {
                            if (popover) {
                                pos = alignSvc.pos(element, popover, align);
                                popover.css({top: pos.top, left: pos.left});
                                popover.addClass('in');
                            }
                        });

                        // Remove popover when clicking it
                        angular.element(popover).click(function () {
                            remove();
                        });
                    }
                }

                // Remove popover from DOM
                function remove() {
                    if (popover) {
                        popover.remove();
                        popover = null;
                    }
                };

                // Destroy popover's scope
                scope.$on('$destroy', function () {
                    remove();
                });

                // Show or hide popover
                function toggle() {
                    if (!popover) {
                        show();
                    }
                    else {
                        remove();
                    }
                }

                if (trigger == 'hover') {
                    angular.element(element).bind("mouseenter", function () {
                        show();
                    });
                    angular.element(element).bind("mouseleave", function () {
                        remove();
                    });
                } else if (trigger == 'click') {
                    angular.element(element).bind("click", toggle);
                }

                // Remove popover when scrolling
                window.onscroll = remove;
                // Remove popover when resizing window
                window.onresize = remove;
                // Remove popover when clicking outside element
                angular.element(document).click(function (e) {
                    if (e.target !== element[0] && !element[0].contains(e.target)) {
                        remove();
                    }
                });
            }
        };
    }]);
