var componentsModules = require('../../components.module');

var popupAddParticipantController = function ($scope, $timeout, ChatUser, ChatAPI, ChatContacts, ChatUtils, ChatMessageLog) {

    // Clear participants already included in the room
    $scope.autocompleteList = ChatUtils.filterList(ChatContacts.list, $scope.popup.participant, "id");

    // Array of contacts to create a conversation group
    $scope.addParticipantList = [];

    // Search contacts in add participant menu
    $scope.searchContacts = ChatContacts.search;

    /**
     * Create a room group
     * @method
     * @params participants
     */
    $scope.createGroup = function () {

        var params = {
            isGroup: true,
            message: null,
            notifyOtherConnections: true,
            roomName: null
        };

        if ($scope.popup.isGroup) {
            $scope.addParticipantToGroup();
        } else {
            // Append first participant to the addParticipantList array (in case user create group without adding current participant)
            $scope.addParticipantList.splice(0, 0, $scope.popup.participant[0]);

            ChatAPI.proxy().server
                .firstContact($scope.addParticipantList, params.message, params.isGroup, params.notifyOtherConnections, params.roomName)
                .then(function (group) {
                    $scope.addParticipantList = [];
                }, function (err) {
                    $timeout(function () {
                        $scope.addParticipantList = [];
                        $scope.toaster.add({
                            code: 3,
                            type: 'error',
                            message: 'Erro ao criar grupo',
                            icon: 'ing ing-exclamation-circle',
                            dismissable: true
                        });
                    }, 0);
                });
        }
    };

    /**
     * Add a new contact to group
     * @method
     * @params participants
     */
    $scope.addParticipantToGroup = function () {
        // Remove participants already added to group
        var newParticipants = [];
        var oldParticipants = [];

        $scope.addParticipantList.forEach(function (newParticipant) {
            if (!$scope.popup.participant.some(function (participant) {
                    return newParticipant.id == participant.id;
                })) {
                newParticipants.push(newParticipant);
            } else {
                oldParticipants.push(newParticipant);
            }
        });

        if (newParticipants.length) {
            ChatAPI.proxy().server
                .addParticipantsToGroup($scope.popup.roomId, newParticipants)
                .then(function () {
                    //    Success
                }, function (err) {
                    $timeout(function () {
                        $scope.toaster.add({
                            code: 3,
                            type: 'error',
                            message: ChatMessageLog.ROOM.ERROR.ADD_PARTICIPANT,
                            icon: 'ing ing-exclamation-circle',
                            dismissable: true
                        });
                    }, 0);
                })
                .always(function () {
                    // Clear added participants array
                    $scope.addParticipantList = [];
                });
        }

        if (oldParticipants.length) {
            oldParticipants.forEach(function (participant) {
                $scope.toaster.add({
                    code: 3,
                    type: 'info',
                    message: participant.name + ' ' + ChatMessageLog.ROOM.ERROR.ALREADY_ADDED,
                    icon: 'ing ing-exclamation-circle',
                    dismissable: true
                });
            })
        }
    };
};

popupAddParticipantController.$inject = ['$scope', '$timeout', 'ChatUser', 'ChatAPI', 'ChatContacts', 'ChatUtils', 'ChatMessageLog'];

componentsModules.directive('chatPopupAddParticipant', function () {
    return {
        restrict: 'AE',
        replace: true,
        templateUrl: 'ingaia-chat/popup/add-participant/popup-add-participant.tpl.html',
        controller: popupAddParticipantController
    }
});
