var appConfig = require("../../config");
var Helpers = require("../../helpers");

/**
 * Merge data from a contact
 * @method
 * @params contact
 */
var mergeConversation = function(dest, source, userId) {
  angular.extend(dest, source);
  // Remove logged user from participant array
  dest.participant = source.participant.filter(function(c) {
    return c.id != userId;
  });
};

var popupController = function(
  $scope,
  $timeout,
  $filter,
  $q,
  ChatUser,
  ChatAPI,
  ChatConversations,
  ChatNavigator,
  ChatMessageLog,
  ChatSidebar,
  ChatPopups,
  ChatContacts,
  ChatToaster,
  ChatUtils,
  Upload
) {
  var User = ChatUser.get();
  var state = $.connection.hub.state;
  var connectionState = $.signalR.connectionState;

  $scope.User = User;
  $scope.popup.messages = []; // Clear old messages to reload last received messages
  $scope.doneMessages = true;
  $scope.toaster = new ChatToaster();
  $scope.TRANSLATE = ChatMessageLog;

  $scope.galleryUrl = "";
  $scope.$watch("popup.roomId", function(roomId) {
    if (roomId) {
      $scope.galleryUrl = appConfig.galleryUrl + "room/" + roomId;
    }
  });

  //UPLOAD***************************************************************/
  $scope.validateUpload = function(validFiles, invalidFiles) {
    if (validFiles.length) {
      $scope.uploadFiles(validFiles);
    }

    invalidFiles.forEach(function(file) {
      var errorText = ChatMessageLog.UPLOAD.ERROR.NOT_SUPPORTED + "<br/>";
      if (file.$errorMessages.pattern) {
        errorText +=
          ChatMessageLog.UPLOAD.ERROR.PATTERN + file.$errorParam.toUpperCase();
      } else if (file.$errorMessages.maxSize) {
        errorText += ChatMessageLog.UPLOAD.ERROR.MAX_SIZE + file.$errorParam;
      } else if (file.$errorMessages.minSize) {
        errorText += ChatMessageLog.UPLOAD.ERROR.MIN_SIZE + file.$errorParam;
      }

      $scope.toaster.add({
        icon: "ing-exclamation-circle",
        type: "error",
        message: errorText,
        dismissable: true
      });
    });
  };

  $scope.uploadValidateOptions = {
    size: { min: "1KB", max: "5MB" },
    pattern:
      ".jpg,.jpeg,.gif,.bmp,.png,.pdf,.doc,.docx,.ppt,.pptx,.pps,.xls,.xlsx,.csv"
  };

  $scope.uploadOptions = {
    debounce: 100
  };

  $scope.uploadFiles = function(files) {
    if ($scope.popup.roomId) {
      sendUploadFiles(files);
    } else {
      var isGroup, notifyConnections, roomName;
      isGroup = false;
      roomName = null;
      notifyConnections = true;

      ChatAPI.proxy()
        .server.firstContact(
          $scope.popup.participant,
          null,
          isGroup,
          notifyConnections,
          roomName
        )
        .done(function(response) {
          mergeConversation($scope.popup, response, User.id);
          $scope.popup.roomId = response.roomId;
          sendUploadFiles(files);
          // Add roomId to opened popups history
          ChatPopups.history().add(response.roomId);
          ChatSidebar.tab.open("conversations");
        })
        .fail(function(err) {
          console.error(err);
        });
    }
  };

  function sendUploadFiles(files) {
    if (files && files.length) {
      angular.forEach(files, function(file) {
        // Temporary message
        var fileExtension = Helpers.file.getFileExtension(file.name);
        var newMessage = angular.copy({
          message: "",
          messageDate: new Date().toISOString(),
          pending: true,
          lastMessage: file.name,
          hasFile: true,
          fileType: fileExtension,
          sender: {
            id: User.id,
            name: User.name,
            photo: User.photo,
            productTag: User.productTag
          }
        });

        $timeout(function() {
          $scope.popup.messages.push(newMessage);
        });

        var onErrorSendUploadMessage = function(err) {
          newMessage.error = true;
          window.inGaiaChat_API.emit("errorSendUploadMessage");
          console.error("Error status: " + err.status);
        };

        var params = {
          roomId: $scope.popup.roomId
        };

        // If file is an image, add width and height params to save in API
        if (file.type.indexOf("image") > -1) {
          params.imageWidth = file.$ngfWidth;
          params.imageHeight = file.$ngfHeight;
        }

        // Generate preview image
        var embedConfig = {
          url: null,
          fileType: fileExtension,
          fileName: file.name
        };

        // Create a sample thumb of the file
        if (Helpers.file.isImage(fileExtension, true)) {
          Upload.dataUrl(file, true).then(function(url) {
            // Create a blob data file
            $timeout(function() {
              embedConfig.url = url;
              newMessage.message = $filter("embedUrl")(embedConfig);
              newMessage.loadedFile = true;
            });
          });
        } else {
          // Just display file type thumbnail
          newMessage.message = $filter("embedUrl")(embedConfig);
          newMessage.loadedFile = true;
        }

        Upload.upload({
          method: "POST",
          url: appConfig.galleryApi,
          data: {
            file: file,
            roomId: $scope.popup.roomId
          },
          headers: {
            Authorization: "Bearer " + ChatAPI.getAPIToken()
          },
          params: params
        }).then(
          function(resp) {
            // Fazer push da mensagem com o anexo
            var uploadResponse = resp.data;
            var messageText = resp.config.data.file.name;

            ChatAPI.proxy()
              .server.sendUploadMessage(
                $scope.popup.roomId,
                messageText,
                uploadResponse.objectKey,
                uploadResponse.fileType
              )
              .done(function(messageResponse) {
                switch (messageResponse.deliveryStatus) {
                  case 3:
                    onErrorSending(ChatMessageLog.MESSAGE.ERROR.SEND);
                    break;
                  case 4:
                    onErrorSending(ChatMessageLog.MESSAGE.ERROR.DELIVERY);
                    break;
                  case 5:
                    onErrorSending(ChatMessageLog.MESSAGE.ERROR.BLOCKED);
                    break;
                }

                // Wait 1 second before remove progress bar (sometimes the upload is too fast to user notice the progress bar)
                $timeout(function() {
                  newMessage.fileObjectKey = uploadResponse.objectKey;
                  newMessage.messageDate = messageResponse.deliveryDate;
                  newMessage.pending = false;
                  newMessage.progress = null;
                }, 1000);

                embedConfig.link =
                  appConfig.galleryUrl +
                  "room/" +
                  $scope.popup.roomId +
                  "/file/" +
                  uploadResponse.objectKey;
                // Rerender file thumb with gallery url
                newMessage.message = $filter("embedUrl")(embedConfig);
                newMessage.loadedFile = true;
              })
              .fail(onErrorSendUploadMessage);
          },
          function(err) {
            onErrorSendUploadMessage(err);
          },
          function(evt) {
            var progressPercentage = parseInt((100.0 * evt.loaded) / evt.total);
            $timeout(function() {
              newMessage.progress = progressPercentage;
            });
          }
        );
      });
    }
  }
  //***************************************************************/

  // Check connection state
  if (state == connectionState.connected) {
    // Get room messages
    if ($scope.popup.roomId) {
      getMessages();
    } else {
      if (!$scope.popup.isGroup) {
        // If is a blokced contact, fill the has blocked property to display unblock message
        $scope.popup.hasBlocked = $scope.popup.participant[0].isBlocked;
        // If is an empty popup, fetch an existent room
        getRoomByParticipant($scope.popup.participant[0]);
      }
    }
  }

  // First Contact from Incorp and Comunidade
  if ($scope.popup.property) {
    var firstContactSubject = "";
    var firstContactId = new Date().getMilliseconds();

    if ($scope.popup.property.message) {
      firstContactSubject = $scope.popup.property.message;
    } else {
      firstContactSubject = "Olá, ";
      firstContactSubject += $scope.popup.participant[0].name + ", tudo bem?\n";
      firstContactSubject += "Possuo clientes com interesse em ";
      firstContactSubject += $scope.popup.property.isEnterprise
        ? "empreendimentos"
        : "imóveis";
      firstContactSubject += " como este: ";
      firstContactSubject +=
        $scope.popup.property.reference +
        ", " +
        $scope.popup.property.neighborhood +
        ", " +
        $scope.popup.property.city;
      firstContactSubject += ". Vamos conversar?";
    }

    $scope.popup.confirm = {
      theme: "light",
      icon: "icon-img icon-first-chat",
      text:
        '<textarea rows="6" class="chat-input-control" id="subject-text-' +
        firstContactId +
        '">' +
        firstContactSubject +
        "</textarea>",
      closeAfterAction: false,
      buttons: [
        {
          label: "Enviar",
          class: "secondary",
          action: function() {
            /* Origins
                            Ficha do imovel: file
                            Radar: radar-parcerias
                            Aba do radar no modal da ficha: radar-tab
                            Incorp: incorp
                        */

            var message = angular
              .element("#subject-text-" + firstContactId)
              .val();
            var firstFrom = parseInt($scope.popup.property.productTag);

            // Create an array of participants (first index should be the message sender)
            var participants = angular.copy($scope.popup.participant);
            // Store the origin of firstContactCustom
            var origin = $scope.popup.property.origin || "file";
            // Add logged user to participants array
            participants.splice(0, 0, User);

            ChatAPI.proxy()
              .server.firstContactCustom(
                participants,
                message,
                firstFrom,
                origin
              )
              .then(
                function(response) {
                  // Clear messages and use getMessages to replace
                  response.messages = [];
                  // Remove property tha flag conversation as First Contact
                  delete $scope.popup.property;
                  mergeConversation($scope.popup, response, User.id);
                  getMessages();
                  // Add roomId to opened popups history
                  ChatPopups.history().add($scope.popup.roomId);
                  $scope.popup.confirm.destroy();
                },
                function(err) {
                  $scope.popup.confirm.error =
                    ChatMessageLog.ROOM.ERROR.PARTNERSHIP;
                }
              );
          }
        }
      ]
    };
  }

  // Check if logged user is group admin
  $scope.isAdmin = function() {
    var isAdmin = false;
    if (
      $scope.popup.admin.id == User.id &&
      $scope.popup.admin.productTag == User.productTag
    )
      isAdmin = true;
    return isAdmin;
  };

  // Close/Open current popup
  $scope.togglePopup = function() {
    $scope.popup.collapsed = !$scope.popup.collapsed;

    if (!$scope.popup.collapsed) $scope.setReadMessages();

    // Update collapse state popup
    ChatPopups.history().update($scope.popup);

    $scope.$emit("togglePopup", $scope.popup.collapsed);
  };

  // Browser support
  $scope.browser = {};
  $scope.browser.supported = ChatNavigator.support.check();

  // Return true if message is from user id
  $scope.isMyMessage = function(senderId) {
    return senderId == User.id;
  };

  function getMessages(params) {
    $scope.doneMessages = false;

    if ($scope.popup.messages == undefined) {
      $scope.popup.messages = [];
    }

    /* If there's a roomId then request messages*/
    if ($scope.popup.roomId) {
      var defaultParams = {
        limit: 20,
        offset: 0,
        userId: User.id,
        productTag: User.productTag
      };

      if (params != undefined) {
        defaultParams.limit = params.limit;
        defaultParams.offset = params.offset;
      }

      if (
        $scope.popup.archives !== null &&
        $scope.popup.archives !== undefined
      ) {
        if ($scope.popup.archives.length) defaultParams.hasArchives = true;
      }

      ChatAPI.getMessages($scope.popup.roomId, defaultParams)
        .then(
          function(response) {
            var messages = response.data;
            if (!messages.length) {
              hasMoreMessages = false;
              return;
            }
            messages.forEach(function(message) {
              $scope.popup.messages.splice(0, 0, message);
            });
          },
          function(err) {
            $scope.toaster.add({
              code: 3,
              type: "error",
              message: ChatMessageLog.MESSAGE.ERROR.LOAD,
              dismissable: true
            });
          }
        )
        .finally(function() {
          $scope.doneMessages = true;
        });
    } else {
      $scope.doneMessages = true;
    }
  }

  function getRoomByParticipant(participant) {
    ChatAPI.proxy()
      .server.getRoomByParticipant(participant.id, participant.productTag)
      .then(
        function(response) {
          if (response) {
            // Replace current popup with found room
            mergeConversation($scope.popup, response);
            getMessages();
          } else {
            $scope.popup.messages = [];
            $scope.doneMessages = true;
          }
        },
        function(err) {
          $scope.toaster.add({
            code: 3,
            type: "error",
            message: ChatMessageLog.MESSAGE.ERROR.LOAD,
            dismissable: true
          });
        }
      );
  }

  var hasMoreMessages = true;
  $scope.loadMoreMessages = function() {
    if (hasMoreMessages && $scope.doneMessages) {
      var params = {
        limit: 20,
        offset: $scope.popup.messages.length
      };
      getMessages(params);
    }
  };

  /* Updates when user is typing */
  var typingTimeout, doneTypingInterval, doneTyping, sentTyping;
  doneTypingInterval = 10000;
  sentTyping = false;

  var clearTypingTimeout = function() {
    sentTyping = false;
    if (typingTimeout !== undefined) clearTimeout(typingTimeout);
  };

  doneTyping = function() {
    sentTyping = true;
    ChatAPI.proxy()
      .server.userIsTyping($scope.popup.roomId, User.status)
      .done(function() {
        typingTimeout = setTimeout(function() {
          sentTyping = false;
        }, doneTypingInterval);
      })
      .fail(function() {
        sentTyping = false;
        clearTypingTimeout();
      });
  };

  /**
   * Notify when someone is typing a message.
   * @method
   * @params event
   */
  $scope.isTyping = function(event) {
    if (!sentTyping) doneTyping();

    // If using emoji-popup, the text field will submit the message without passing by here
    if (event.keyCode == 13 && !event.shiftKey) {
      clearTypingTimeout();
      if (!$scope.isSupported) sendMessage();
    }
  };

  // Send message (when using emoji directive)
  $scope.emojiMessage = {};
  $scope.emojiMessage.replyToUser = function() {
    sendMessage();
  };

  /**
   * Send message to room
   * @method
   */
  var sendMessage = function() {
    clearTypingTimeout();

    if (!$scope.popup.isGroup && $scope.popup.participant[0].isBlocked) {
      $scope.toaster.add({
        code: 3,
        type: "error",
        message: ChatMessageLog.MESSAGE.ERROR.BLOCKED,
        dismissable: true
      });
      return false;
    }

    var charLimit = 10000;
    var newMessageText = $filter("stripTags")(
      $scope.emojiMessage.messagetext
    ).substr(0, charLimit);
    newMessageText = $filter("colonToCode")(newMessageText); // Convert to colonToCode for native apps

    if (!newMessageText.trim().length) {
      return;
    }

    // Temporary message
    var newMessage = {
      message: newMessageText,
      messageDate: new Date().toISOString(),
      pending: true,
      sender: {
        id: User.id,
        name: User.name,
        photo: User.photo,
        productTag: User.productTag
      }
    };

    if ($scope.popup.messages == undefined) {
      $scope.popup.messages = [];
    }

    $scope.popup.messages.push(newMessage);

    var onErrorSending = function(err) {
      newMessage.error = true;
      window.inGaiaChat_API.emit("errorSendMessage");
    };

    if ($scope.popup.roomId) {
      $scope.setReadMessages();
      // New conversation WITH roomId
      ChatAPI.proxy()
        .server.sendMessage($scope.popup.roomId, newMessageText)
        .done(function(response) {
          $timeout(function() {
            switch (response.deliveryStatus) {
              case 3:
                onErrorSending(ChatMessageLog.MESSAGE.ERROR.SEND);
                break;
              case 4:
                onErrorSending(ChatMessageLog.MESSAGE.ERROR.DELIVERY);
                break;
              case 5:
                onErrorSending(ChatMessageLog.MESSAGE.ERROR.BLOCKED);
                break;
            }
            newMessage.messageDate = response.deliveryDate;
            newMessage.pending = false;
          }, 0);
        })
        .fail(onErrorSending);
      // Clear text field
      $scope.emojiMessage.messagetext = "";
    } else {
      // If user is from contact list, send first contact request
      var isGroup, notifyConnections, roomName;
      isGroup = false;
      roomName = null;
      notifyConnections = true;

      ChatAPI.proxy()
        .server.firstContact(
          $scope.popup.participant,
          newMessageText,
          isGroup,
          notifyConnections,
          roomName
        )
        .done(function(response) {
          mergeConversation($scope.popup, response, User.id);
          // Add roomId to opened popups history
          newMessage.messageDate = response.lastMessageDate;
          newMessage.pending = false;
          ChatPopups.history().add(response.roomId);
          ChatSidebar.tab.open("conversations");
        })
        .fail(onErrorSending);
    }
  };

  $scope.sendMessage = sendMessage;

  /**
   * Set all messages on current popup as read
   * @method
   */
  $scope.setReadMessages = function() {
    if ($scope.popup.roomId && $scope.popup.unreadMessages > 0) {
      ChatAPI.proxy()
        .server.setUnreadMessagesAsRead($scope.popup.roomId)
        .then(
          function() {
            $timeout(function() {
              ChatConversations.unreads -= $scope.popup.unreadMessages;
              $scope.popup.unreadMessages = 0;
              window.inGaiaChat_API.emit(
                "chatNotificationUpdate",
                ChatConversations.unreads
              );
              window.inGaiaChat_API.emit(
                "setReadMessages",
                $scope.popup.roomId
              );
            }, 0);
          },
          function(err) {
            console.error(err);
          }
        );
    }
  };

  // Highlight send button (mobile screens)
  var sendBtn = angular.element(".send-message-button");

  $scope.$watch("emojiMessage.messagetext", function(message) {
    if (message && message != "") {
      if (message.length > 0) sendBtn.addClass("active");
      else sendBtn.removeClass("active");
    }
  });

  // TABS =======================================
  $scope.activeTab = "messages";

  $scope.setActiveTab = function(tab) {
    $scope.activeTab = $scope.activeTab == tab ? "" : tab;
  };

  $scope.isActiveTab = function(tab) {
    return tab == $scope.activeTab ? true : false;
  };

  /**
   * Confirm action Delete group
   * @method
   * @params roomId
   */
  $scope.confirmDeleteConversation = function() {
    $scope.popup.confirm = {
      icon: "ing ing-exclamation-circle",
      title: ChatMessageLog.ROOM.DELETE.TITLE,
      text: ChatMessageLog.ROOM.DELETE.TEXT,
      buttons: [
        {
          label: ChatMessageLog.ACTIONS.CANCEL,
          class: "secondary",
          action: function() {}
        },
        {
          label: ChatMessageLog.ACTIONS.DELETE,
          class: "primary",
          action: deleteConversation
        }
      ]
    };
  };

  /**
   * Delete a room group
   * @method
   * @params roomId
   */
  var deleteConversation = function() {
    var isGroup = $scope.popup.isGroup;
    ChatAPI.proxy()
      .server.archive($scope.popup.roomId, User.id, User.productTag, isGroup)
      .then(
        function() {
          $scope.closePopup($scope.popup);
          window.inGaiaChat_API.emit(
            "chatPopupRoomExcluded",
            $scope.popup.roomId
          );
        },
        function(err) {
          $scope.setActiveTab("messages");
          $scope.toaster.add({
            code: 3,
            type: "error",
            message: ChatMessageLog.ROOM.ERROR.DELETE,
            dismissable: true
          });
        }
      );
  };

  // Close current popup
  $scope.closePopup = function(popup) {
    $timeout(function() {
      // Clear confirm messages
      $scope.popup.confirm = null;

      // Reset block value if user close popup without confirming
      if (!$scope.popup.isGroup)
        $scope.popup.hasBlocked = $scope.popup.participant[0].isBlocked;

      // Close popup and destroy scope
      ChatPopups.closePopup(popup);
      $scope.$broadcast("closePopup");
      $scope.$destroy();
    }, 250);
  };

  // Event Listeners
  var onReceiveMessage = function(data) {
    if (data.roomId == $scope.popup.roomId) {
      $scope.$emit("receiveMessage");
      if (!$scope.popup.messages) $scope.popup.messages = [];
      $scope.popup.messages.push(data.message);
    }
  };

  var onUpdateRoomUnreads = function(data) {
    if (data.roomId == $scope.popup.roomId) {
      $scope.popup.unreadMessages = data.unreadMessages;
    }
  };

  var onUserChangeStatus = function(data) {
    $scope.popup.participant.forEach(function(participant) {
      if (
        participant.id == data.id &&
        participant.productTag == data.productTag
      ) {
        $timeout(function() {
          participant.status = data.status;
        }, 0);
      }
    });
  };

  var onMessageAsRead = function(roomId) {
    if (roomId == $scope.popup.roomId) {
      $timeout(function() {
        $scope.popup.messages.forEach(function(message) {
          if (
            message.sender.id == User.id &&
            message.sender.productTag == User.productTag
          )
            message.read = true;
        });
      }, 0);
    }
  };

  var onRoomEdited = function(room) {
    if (room.roomId == $scope.popup.roomId) {
      $scope.popup.name = room.name;
    }
  };

  var onReconnected = function() {
    // Refresh messages to list latest messages that could be arrived during disconnection
    $scope.popup.messages = [];
    if ($scope.popup.roomId) {
      getMessages();
    } else {
      getRoomByParticipant($scope.popup.participant[0]);
    }
  };

  var onRoomExcluded = function(roomId) {
    if (roomId == $scope.popup.roomId) {
      $scope.closePopup($scope.popup);
    }
  };

  // Bind Chat Listeners
  window.inGaiaChat_API.on("chatUpdateRoomUnreads", onUpdateRoomUnreads);
  window.inGaiaChat_API.on("chatMessageAsRead", onMessageAsRead);
  window.inGaiaChat_API.on("chatUserChangeStatus", onUserChangeStatus);
  window.inGaiaChat_API.on("chatReceiveMessage", onReceiveMessage);
  window.inGaiaChat_API.on("chatRoomEdited", onRoomEdited);
  window.inGaiaChat_API.on("chatReconnected", onReconnected);
  window.inGaiaChat_API.on("chatRoomExcluded", onRoomExcluded);

  $scope.$on("$destroy", function() {
    //Unbind Chat Listeners
    window.inGaiaChat_API.off("chatUpdateRoomUnreads", onUpdateRoomUnreads);
    window.inGaiaChat_API.off("chatMessageAsRead", onMessageAsRead);
    window.inGaiaChat_API.off("chatUserChangeStatus", onUserChangeStatus);
    window.inGaiaChat_API.off("chatReceiveMessage", onReceiveMessage);
    window.inGaiaChat_API.off("chatRoomEdited", onRoomEdited);
    window.inGaiaChat_API.off("chatReconnected", onReconnected);
    window.inGaiaChat_API.off("chatRoomExcluded", onRoomExcluded);
  });
};

popupController.$inject = [
  "$scope",
  "$timeout",
  "$filter",
  "$q",
  "ChatUser",
  "ChatAPI",
  "ChatConversations",
  "ChatNavigator",
  "ChatMessageLog",
  "ChatSidebar",
  "ChatPopups",
  "ChatContacts",
  "ChatToaster",
  "ChatUtils",
  "Upload"
];

var popupLink = function(scope, element) {
  angular.element(element).addClass("popup-enter");

  var enableTextField = function(enable) {
    var textField = angular
      .element(element)[0]
      .querySelector(".emoji-wysiwyg-editor");
    if (textField) {
      if (enable) {
        textField.setAttribute("contenteditable", true);
      } else {
        textField.setAttribute("contenteditable", false);
      }
    }
  };

  var focusTextField = function() {
    var textField = angular
      .element(element)[0]
      .querySelector(".emoji-wysiwyg-editor");
    if (textField) {
      angular
        .element(element)[0]
        .querySelector(".emoji-wysiwyg-editor")
        .focus();
      scope.setReadMessages();
    }
  };

  var state = $.connection.hub.state;
  var connectionState = $.signalR.connectionState;

  setTimeout(function() {
    if (
      state == connectionState.disconnected ||
      state == connectionState.connecting
    ) {
      onDisconnected();
    } else {
      // Remove animation class after display popup
      angular.element(element).removeClass("slide-in-up");

      // Avoid focus on text field when selecting text
      var $messagesBox = element[0].querySelector(".direct-chat-messages");
      if ($messagesBox) {
        $messagesBox.addEventListener("click", function(event) {
          var selection = window.getSelection();
          if (selection.type != "Range") {
            focusTextField();
          }
        });
      }

      // Focus on text field if popup starts opened
      if (!scope.popup.collapsed) {
        focusTextField();
      }
    }

    angular.element(element).removeClass("popup-enter");
  }, 500);

  // Event Listeners
  var onDisconnected = function() {
    scope.toaster.add({
      icon: "ing-exclamation-circle",
      code: 3,
      type: "error",
      message: "Você está desconectado",
      dismissable: false
    });
    enableTextField(false);
  };

  var onReconnected = function() {
    scope.toaster.removeAll();
    enableTextField(true);
  };

  window.inGaiaChat_API.on("chatDisconnected", onDisconnected);
  window.inGaiaChat_API.on("chatReconnected", onReconnected);

  scope.$on("closePopup", function() {
    angular.element(element).addClass("slide-out-down popup-leave");
    window.inGaiaChat_API.off("chatDisconnected", onDisconnected);
    window.inGaiaChat_API.off("chatReconnected", onReconnected);
  });

  // Set message as read if current popup is focused when receiving a message
  scope.$on("receiveMessage", function() {
    if (
      angular.element(document.activeElement)[0] ==
      angular.element(element)[0].querySelector(".emoji-wysiwyg-editor")
    ) {
      setTimeout(function() {
        scope.setReadMessages();
      }, 1000);
    }
  });

  scope.$on("togglePopup", function(event, collapsed) {
    if (!collapsed) {
      setTimeout(focusTextField, 300);
    }
  });
};

var componentsModules = require("../components.module");
componentsModules.directive("chatPopup", function() {
  return {
    restrict: "EA",
    templateUrl: "ingaia-chat/popup/popup.tpl.html",
    replace: true,
    scope: {
      popup: "=chatPopup"
    },
    controllerAs: "vm",
    controller: popupController,
    link: popupLink
  };
});
