var componentsModules = require('../../components.module');

var popupSettingsGroupController = function ($scope, $timeout, ChatUser, ChatAPI, ChatContacts, ChatUtils, ChatMessageLog) {

    var User = ChatUser.get();

    if ($scope.popup.isGroup) {
        mergeBlocked();
    }

    function mergeBlocked() {
        $timeout(function () {
            ChatUtils.mergeBlockeds($scope.popup.participant, ChatContacts.blocked);
        }, 1000);
    };

    /**
     * Edit room group information
     * @method
     * @params room
     */
    $scope.editRoomName = function (name) {
        if (name.length > 25) {
            name = name.substr(0, 25);
        }

        ChatAPI.proxy().server
            .editRoomName($scope.popup.roomId, name)
            .then(function () {
                    $timeout(function () {
                        $scope.popup.name = name;
                        $scope.popup.tempName = '';
                        $scope.setActiveTab('messages');
                    }, 0);
                },
                function (err) {
                    $scope.setActiveTab('messages');
                    $scope.toaster.add({
                        code: 3,
                        type: 'error',
                        message: ChatMessageLog.ROOM.ERROR.EDIT,
                        dismissable: true
                    });
                });
    };

    /**
     * Confirm sign out from group action
     * @method
     * @params roomId
     */
    $scope.confirmLeaveGroup = function () {
        $scope.popup.confirm = {
            icon: 'ing ing-exclamation-circle',
            title: ChatMessageLog.ROOM.LEAVE.TITLE,
            text: ChatMessageLog.ROOM.LEAVE.TEXT,
            buttons: [
                {
                    label: ChatMessageLog.ACTIONS.STAY,
                    class: 'secondary',
                    action: function () {
                    }
                },
                {
                    label: ChatMessageLog.ACTIONS.LEAVE,
                    class: 'primary',
                    action: function () {
                        leaveGroup();
                    }
                }
            ]
        };
    };

    /**
     * Sign Out From Group
     * @method
     */
    var leaveGroup = function () {
        ChatAPI.proxy().server
            .removeParticipantsFromGroup($scope.popup.roomId, [{
                id: User.id,
                name: User.name,
                productTag: User.productTag
            }], true)
            .then(function () {
                    $scope.closePopup($scope.popup);
                },
                function (err) {
                    $scope.setActiveTab('messages');
                    $scope.toaster.add({
                        code: 3,
                        type: 'error',
                        message: ChatMessageLog.ROOM.ERROR.LEAVE,
                        dismissable: true
                    });
                });
    };
    // End Sign Out From Group ========================

    /**
     * Add a new contact to group
     * @method
     * @params participants
     */
    $scope.removeParticipantsFromGroup = function (participant) {
       
        $timeout(function() {
            participant.deleting = true;
        }, 0);

        var list = [];

        if (Array.isArray(participant))
            list = participant;
        else
            list.push(participant);

        ChatAPI.proxy().server
            .removeParticipantsFromGroup($scope.popup.roomId, list, false)
            .then(function () {
                //    Success
            }, function (err) {
                $scope.setActiveTab('messages');
                $scope.toaster.add({
                    code: 3,
                    type: 'error',
                    message: ChatMessageLog.ROOM.ERROR.REMOVE_PARTICIPANT,
                    dismissable: true
                });
            });
    };

    window.inGaiaChat_API.on('chatContactBlockChange', mergeBlocked);

    $scope.$on('$destroy', function () {
        //Unbind Chat Listeners
        window.inGaiaChat_API.off('chatContactBlockChange', mergeBlocked);
    });
};

popupSettingsGroupController.$inject = ['$scope', '$timeout', 'ChatUser', 'ChatAPI', 'ChatContacts', 'ChatUtils', 'ChatMessageLog'];

componentsModules.directive('chatPopupSettingsGroup', function () {
    return {
        restrict: 'AE',
        replace: true,
        templateUrl: 'ingaia-chat/popup/settings-group/popup-settings-group.tpl.html',
        controller: popupSettingsGroupController
    }
});
