var componentsModules = require('../../components.module');

var popupSettingsSingleController = function ($timeout, $scope, ChatUser, ChatAPI, ChatMessageLog, ChatContacts) {

    var User = ChatUser.get();

    /**
     * Toggle block contact action
     * @method
     * @params isBlocking
     */
    $scope.toggleBlock = function (isBlocking) {
        $scope.confirmBlockContact(isBlocking);
    };

    /**
     * Confirm block contact action
     * @method
     * @params roomId
     */
    $scope.confirmBlockContact = function (isBlocking) {
        isBlocking = (isBlocking != undefined) ? isBlocking : false;
        // If previous block state if false, then user is trying to block
        $scope.popup.confirm = {
            icon: 'ing ing-exclamation-circle',
            title: (isBlocking) ? ChatMessageLog.BLOCK.CONFIRM : ChatMessageLog.UNBLOCK.CONFIRM,
            text: (isBlocking) ? ChatMessageLog.BLOCK.WARN : ChatMessageLog.UNBLOCK.WARN,
            buttons: [
                {
                    label: ChatMessageLog.ACTIONS.CANCEL,
                    class: 'secondary',
                    action: function () {
                        $scope.popup.hasBlocked = !isBlocking;
                    }
                },
                {
                    label: ((isBlocking) ? ChatMessageLog.ACTIONS.BLOCK : ChatMessageLog.ACTIONS.UNBLOCK),
                    class: 'primary',
                    action: function () {
                        if (isBlocking)
                            blockContact();
                        else
                            unblockContact();
                    }
                }
            ]
        };
    };

    /**
     * Block a contact
     * @method
     * @params contact
     */
    var blockContact = function () {
        var participant = $scope.popup.participant[0];

        var successBlock = function () {
            $scope.setReadMessages();
            $scope.closePopup($scope.popup);
        };

        var errorBlock = function (err) {
            // Rollback default block value
            $scope.popup.hasBlocked = participant.isBlocked = false;
            $scope.toaster.add({
                type: 'error',
                message: ChatMessageLog.BLOCK.ERROR,
                dismissable: true
            });
        };

        ChatContacts.block(participant, successBlock, errorBlock);
    };

    /**
     * Block a contact
     * @method
     * @params contact
     */
    var unblockContact = function () {
        var currentParticipant = $scope.popup.participant[0];
        
        var errorUnblock = function (err) {
            // Rollback default block value
            $scope.popup.hasBlocked = currentParticipant.isBlocked = true;
            $scope.toaster.add({
                type: 'error',
                message: ChatMessageLog.UNBLOCK.ERROR,
                dismissable: true
            });
        };

        ChatContacts.unblock(currentParticipant, null, errorUnblock);
    };
};

popupSettingsSingleController.$inject = ['$timeout', '$scope', 'ChatUser', 'ChatAPI', 'ChatMessageLog', 'ChatContacts'];

componentsModules.directive('chatPopupSettingsSingle', function () {
    return {
        restrict: 'AE',
        replace: true,
        templateUrl: 'ingaia-chat/popup/settings-single/popup-settings-single.tpl.html',
        controller: popupSettingsSingleController
    }
});
