var componentsModules = require("../components.module");
componentsModules.directive("chatTagsInput", [
  "ChatUtils",
  "ChatUser",
  function(ChatUtils, ChatUser) {
    return {
      restrict: "E",
      transclude: true,
      scope: {
        taglist: "=?taglist",
        search: "=?search",
        autocomplete: "=?autocomplete"
      },
      templateUrl: "ingaia-chat/tags-input/tags-input.tpl.html",
      replace: true,
      link: function(scope, element, attrs) {
        var loggedUser = ChatUser.get();

        var withoutLoggedUser = function(list) {
          list = list || [];
          return list.filter(function(item) {
            return item.id !== loggedUser.id;
          });
        };

        scope.defaultWidth = 100;
        scope.tagText = "";
        scope.placeholder = attrs.placeholder;
        scope.taglist = scope.taglist || [];
        scope.autocomplete = withoutLoggedUser(scope.autocomplete);
        scope.pendingSearch = false;
        scope.tags = tagArray();

        var searchDebounced = ChatUtils.debounce(searchTerm, 300);

        scope.toggleTag = function(tag) {
          if (scope.isInArray(tag.id)) scope.deleteTag(tag.id);
          else scope.addTag(tag);

          scope.tags = tagArray();
        };

        scope.isInArray = function(key) {
          return scope.tags.some(function(tag) {
            return tag.id == key;
          });
        };

        scope.addTag = function(tag) {
          scope.taglist.push(tag);
          scope.tags = tagArray();
        };

        scope.deleteTag = function(key) {
          for (var i = 0; i < scope.taglist.length; i++) {
            if (scope.taglist[i].id == key) {
              scope.taglist.splice(i, 1);
            }
          }
          scope.tags = tagArray();
        };

        scope.searchresults = [];

        scope.$watch("tagText", function(newVal) {
          searchDebounced(newVal);
        });

        var $inputContainer = element[0].querySelector(".tag-input-ctn");
        var $inputText = $inputContainer.querySelector("input");

        // Focus no ao carregar componente
        $inputText.focus();

        $inputContainer.addEventListener("click", function() {
          $inputText.focus();
        });

        $inputText.addEventListener("keydown", function(e) {
          var key = e.which;

          if (key === 9 || key === 13) {
            e.preventDefault();
          }

          if (key === 8) {
            if (scope.taglist.length && scope.tagText == "") {
              var last = scope.tags[scope.tags.length - 1];
              if (last.toDelete) scope.deleteTag(last.id);
              else last.toDelete = true;
            }
          } else {
            if (scope.taglist.length && scope.tagText == "") {
              var last = scope.tags[scope.tags.length - 1];
              if (last.toDelete) delete last.toDelete;
            }
          }
        });

        function tagArray() {
          var list = scope.taglist.map(function(tag) {
            return { id: tag.id, name: tag.name };
          });
          return list;
        }

        function searchTerm(term) {
          if (term && term.length) {
            if (!!scope.search) {
              scope.pendingSearch = true;
              scope.search(term).then(
                function(response) {
                  scope.searchresults = withoutLoggedUser(response);
                  scope.pendingSearch = false;
                },
                function() {
                  scope.pendingSearch = false;
                }
              );
            }
          } else {
            scope.searchresults = [];
          }
        }
      }
    };
  }
]);
