var factoriesModules = require("../factories.module");
factoriesModules.factory("ChatConversations", [
  "ChatAPI",
  "ChatUser",
  function(ChatAPI, ChatUser) {
    var defaults = {
      limit: 20,
      offset: 0
    };

    /**
     * Return an Array of conversations.
     * @method
     * @param {Object} options - {id, limit, offset}
     * @return {Array}
     */
    var getConversations = function(options, onsuccess, onerror) {
      var self = this;
      var defaultsOptions;
      var User = ChatUser.get();

      defaultsOptions = {
        id: User.id,
        limit: self.defaults.limit,
        offset: self.defaults.offset
      };

      if (options != null || options != undefined) {
        angular.extend(defaultsOptions, options);
      }

      try {
        self.pending = true;
        ChatAPI.getConversations(defaultsOptions)
          .then(
            function(response) {
              self.defaults.offset = defaultsOptions.offset;
              self.loadMore = response.data.data.length ? true : false;
              var conversations = response.data.data;

              if (onsuccess) {
                onsuccess(conversations);
              } else {
                self.list = conversations;
              }
            },
            function(err) {
              if (onerror != undefined) {
                onerror(err);
              }
              console.error(err.statusText);
            }
          )
          .finally(function() {
            self.pending = false;
          });
      } catch (err) {
        console.error(err);
      }
    };

    var existConversation = function(roomId, returnObject) {
      returnObject = returnObject || false;
      var obj = {};
      var bool = false;
      for (var i = 0; i < this.list.length; i++) {
        if (roomId == this.list[i].roomId) {
          bool = true;
          obj = this.list[i];
          break;
        }
      }
      return returnObject ? obj : bool;
    };

    return {
      defaults: defaults,
      unreads: 0,
      list: [],
      pending: false,
      count: null,
      loadMore: true,
      getConversations: getConversations,
      exist: existConversation
    };
  }
]);
