var factoriesModules = require("../factories.module");
factoriesModules.factory("ChatNavigator", [
  "ChatMessageLog",
  function(ChatMessageLog) {
    return {
      audio: {
        src: "https://chat.ingaia.com.br/assets/audio/chat-notification",
        file: null,
        check: function() {
          // Instanciate a Audio if doesn't exist
          if (this.file == null) this.file = new Audio();

          // Return true if audio was already checked
          if (this.file.src != "") return true;

          // Check audio file support
          if (
            this.file.canPlayType("audio/wav") === "probably" ||
            this.file.canPlayType("audio/wav") === "maybe"
          ) {
            this.file.src = this.src + ".wav";
            return true;
          } else if (
            this.file.canPlayType("audio/ogg") === "probably" ||
            this.file.canPlayType("audio/ogg") === "maybe"
          ) {
            this.file.src = this.src + ".ogg";
            return true;
          } else if (
            this.file.canPlayType("audio/mp3") === "probably" ||
            this.file.canPlayType("audio/mp3") === "maybe"
          ) {
            this.file.src = this.src + ".mp3";
            return true;
          } else {
            return false;
          }
        },
        play: function() {
          var canPlay = this.check();
          if (canPlay) this.file.play();
          else console.error(ChatMessageLog.NAVIGATOR.ERROR.AUDIO);
        }
      },
      support: {
        check: function() {
          var N = navigator.appName,
            ua = navigator.userAgent,
            tem;
          var M = ua.match(
            /(opera|chrome|safari|firefox|msie|trident)\/?\s*(\.?\d+(\.\d+)*)/i
          );
          if (M && (tem = ua.match(/version\/([\.\d]+)/i)) != null) {
            M[2] = tem[1];
          }
          M = M ? [M[1], M[2]] : [N, navigator.appVersion, "-?"];

          if (
            M[0] === "MSIE" ||
            M[0] === "Trident" ||
            /Edge/.test(navigator.userAgent)
          ) {
            if (
              (M[0] === "Trident" && M[1] < 7) ||
              (M[0] === "MSIE" && M[1] < 9)
            ) {
              Errors.setError(1, ChatMessageLog.ERROR.BROWSER_SUPPORT);
              return;
            }
            M.supported = false;
          } else {
            M.supported = true;
          }
          return M;
        }
      },
      notification: {
        enabled: false,
        askPermission: function() {
          if (Notification.permission !== "granted") {
            Notification.requestPermission();
          }
          if (Notification.permission == "granted") {
            this.enabled = true;
          }
        },
        notify: function(contact, message, callback) {
          if (!Notification) {
            console.warn(
              "Desktop notifications not available in your browser."
            );
            return;
          }
          if (Notification.permission == "granted") {
            var notification;
            notification = new Notification(contact.name, {
              icon: contact.photo,
              body: message,
              tag: "message-notify"
            });

            notification.onclick = function(event) {
              event.preventDefault();
              window.focus();
            };

            if (callback != undefined) notification.onclick = callback;
          } else {
            this.askPermission();
          }
        }
      }
    };
  }
]);
