var factoriesModules = require('../factories.module');
factoriesModules.factory('ChatPopups', ['$rootScope', '$timeout', 'ChatConversations', 'ChatSidebar', 'ChatCookies', 'ChatStorage', function ($rootScope, $timeout, ChatConversations, ChatSidebar, ChatCookies, ChatStorage) {
    return {
        list: [],
        grouped: [],
        groupedUnreads: 0,
        limit: 5,
        hasEnoughArea: function () {

            if (this.list.length == 0)
                return true;

            // Check if there is screen space to open a new popup
            var docW, sideW, popW, popCount, groupTabW, firstPopupElem, areaAvailable, defaultWidth, gutter;

            firstPopupElem = document.getElementsByClassName('chat-popup')[0];
            defaultWidth = 275;
            gutter = 15;
            docW = document.getElementsByTagName('body')[0].offsetWidth;
            sideW = document.getElementsByClassName('chat-sidebar')[0].offsetWidth;
            popW = (firstPopupElem) ? firstPopupElem.offsetWidth + gutter : defaultWidth + gutter;
            popCount = this.list.length;
            groupTabW = 0;

            if (this.grouped.length) {
                try {
                    groupTabW = document.getElementsByClassName('popup-group')[0].offsetWidth;
                } catch (err) {
                    return false;
                }
            }

            // Calculate available area
            areaAvailable = Math.floor((docW - sideW) - ((popW * popCount) + groupTabW));

            if (areaAvailable > popW)
                return true;
            else
                return false;
        },
        highlight: function (popup) {
            popup.highlight = true;
            $timeout(function () {
                delete popup.highlight;
            }, 1000);
        },
        shift: function (popup) {
            this.closePopup(popup);
            this.openPopup(popup);
        },
        isOpen: function (contact, shouldHighlight) {
            shouldHighlight = (shouldHighlight != undefined) ? shouldHighlight : true;
            var opened = false;
            var isGroup = (contact.isGroup) ? true : false;

            // Check if popup is inside grouped tab
            if (this.grouped.length) {
                for (var i = 0; i < this.grouped.length; i++) {
                    if (isGroup) {
                        if (contact.roomId == this.grouped[i].roomId) {
                            opened = true;
                            window.inGaiaChat_API.emit('popupGroupedHighlight');
                            break;
                        }
                    } else if (contact.roomId) {
                        if (this.grouped[i].roomId) {
                            if (contact.roomId == this.grouped[i].roomId) {
                                opened = true;
                                window.inGaiaChat_API.emit('popupGroupedHighlight');
                                break;
                            }
                        } else if (contact.participant[0].id == this.grouped[i].participant[0].id) {
                            opened = true;
                            window.inGaiaChat_API.emit('popupGroupedHighlight');
                            break;
                        }
                    } else {
                        if (contact.id == this.grouped[i].participant[0].id && !this.grouped[i].isGroup) {
                            opened = true;
                            window.inGaiaChat_API.emit('popupGroupedHighlight');
                            break;
                        }
                    }
                }
                if (opened)
                    return opened;
            }

            //Check if popup is in popup opened list
            for (var i = 0; i < this.list.length; i++) {
                if (isGroup) {
                    if (contact.roomId == this.list[i].roomId) {
                        opened = true;
                        if (shouldHighlight)
                            this.highlight(this.list[i]);
                        break;
                    }
                } else if (contact.roomId) {
                    if (this.list[i].roomId) {
                        if (contact.roomId == this.list[i].roomId) {
                            opened = true;
                            if (shouldHighlight)
                                this.highlight(this.list[i]);
                            break;
                        }
                    } else if (contact.participant[0].id == this.list[i].participant[0].id) {
                        opened = true;
                        if (shouldHighlight)
                            this.highlight(this.list[i]);
                        break;
                    }
                } else {
                    if (contact.id == this.list[i].participant[0].id && !this.list[i].isGroup) {
                        opened = true;
                        if (shouldHighlight)
                            this.highlight(this.list[i]);
                        break;
                    }
                }
            }

            return opened;
        },
        getGroupUnreads: function () {
            var self = this;
            if (self.grouped.length) {
                self.groupedUnreads = 0;
                self.grouped.forEach(function (contact) {
                    self.groupedUnreads += contact.unreadMessages;
                });
                return self.groupedUnreads;
            }
        },
        openPopup: function (contact, collapsed, property) {
            var self = this;
            collapsed = (collapsed != undefined) ? collapsed : false;
            
            // Check if current contact has a popup
            if (self.isOpen(contact)) return;

            if (self.limit <= self.list.length + self.grouped.length) {
                if (self.grouped.length) {
                    self.closePopup(self.grouped[0]);
                } else {
                    var last = self.list.length - 1;
                    self.closePopup(self.list[last]);
                }
            }

            // If there's no roomId, contact is from contact list
            // Contacts list
            if (!contact.roomId) {
                var newPopup = {};
                newPopup.participant = [contact];
                if (property)
                    newPopup.property = property;
                contact = newPopup;
            }

            contact.collapsed = collapsed;

            // Check if current popup is already open
            if (self.hasEnoughArea()) {
                self.list.splice(0, 0, contact);
            } else {
                // Remove first popup from popups array
                var lastIndex = self.list.length - 1;
                var lastPopup = self.list.splice(lastIndex, 1)[0];
                // Append new popup to popups array
                self.list.splice(0, 0, contact);
                // Append popup removed to popupsGroupedTemp
                self.grouped.push(lastPopup);
            }

            if (contact.roomId)
                self.history().add(contact.roomId);

            window.inGaiaChat_API.emit('popupOpened', contact);

            // Adds class flag indicating popover opened
            angular.element('body').addClass('has-chat-popup-active');
        },
        closePopup: function (popup) {

            var popupIndex = this.list.indexOf(popup);

            if (popupIndex > -1) {
                this.list.splice(popupIndex, 1);
                // Se houver algum popup agrupado, inserir o primeiro agrupado na ultima posicao dos abertos
                if (this.grouped.length) {
                    var firstGrouped = this.grouped.splice(0, 1)[0];
                    this.list.push(firstGrouped);
                }
            }

            var groupedIndex = this.grouped.indexOf(popup);
            if (groupedIndex > -1) {
                this.grouped.splice(groupedIndex, 1);
            }

            if (popup.roomId)
                this.history().remove(popup.roomId);

            this.updateIndex();

            // Remove body class flag when there's no more popover opened
            if (this.list.length <= 1)
                angular.element(document.getElementsByTagName('body')).removeClass('has-chat-popup-active');
        },
        history: function () {
            var self = this;
            var popups_history;
            var webapi_token = ChatCookies.read('webapi_token');

            ChatStorage.get('popups_history', function (data) {
                popups_history = data;
            });

            var add = function (val, collapsed) {                
                if (val == null || val == undefined)
                    return;

                collapsed = (collapsed != undefined) ? collapsed : false;

                if (webapi_token != "") {
                    var popup = {
                        id: val,
                        collapsed: collapsed
                    };

                    if (popups_history) {
                        var exists = popups_history.popups.some(function(p){
                            return p.id == val;
                        });
                        if (!exists)
                            popups_history.popups.push(popup);
                    } else {
                        popups_history = {
                            token: webapi_token,
                            popups: [popup]
                        };
                    }

                    ChatStorage.set({
                        popups_history: popups_history
                    });
                }
            };

            var remove = function (val) {

                if (val == null || val == undefined)
                    return

                if (popups_history) {
                    var popups = popups_history.popups.filter(function(p){
                        return p.id != val;
                    });

                    popups_history.popups = popups;

                    ChatStorage.set({
                        popups_history: popups_history
                    });
                }
            };

            var open = function () {

                if (webapi_token != popups_history.token) {
                    ChatStorage.remove('popups_history');
                    return;
                }

                if (popups_history) {
                    popups_history.popups.forEach(function (h) {
                        ChatConversations.list.forEach(function (c) {
                            if (h.id == c.roomId) {
                                $timeout(function () {
                                    c.collapsed = h.collapsed;
                                    self.openPopup(c, c.collapsed);
                                }, 500);
                            }
                        });
                    });
                }
            };

            var update = function(popup){
                if (popups_history) {
                    popups_history.popups.map(function(p){
                        if(popup.roomId && popup.roomId == p.id) {
                            p.collapsed = popup.collapsed;                        
                        }
                    });

                    ChatStorage.set({
                        popups_history: popups_history
                    });
                };
            };

            var clear = function () {
                ChatStorage.remove('popups_history');
            };

            return {
                open: open,
                add: add,
                remove: remove,
                update: update,
                clear: clear
            };
        },
        updateIndex: function () {
            this.list.forEach(function (popup, index) {
                popup.index = index;
            })
        }
    }
}]);
