var chatSidebarFactory = function ($rootScope, $timeout, ChatAPI, ChatUser, ChatStatus, ChatMessageLog, ChatErrors) {

    var Errors = new ChatErrors;

    var search = {
        text: "",
        isOpen: false,
        open: function () {
            this.isOpen = true;
        },
        close: function () {
            this.isOpen = false;
        },
        toggle: function () {
            this.isOpen = !this.isOpen;
            ChatStatus.close();
        }
    };

    var tab = {
        default: 'conversations',
        current: null,
        init: function () {
            this.current = this.default;
            return this;
        },
        open: function (tab) {
            this.current = tab || this.default;
            ChatStatus.close();
        },
        toggle: function (tab) {
            if (this.current == tab) {
                this.current = this.default; // Toggle tab if it is already active
            } else {
                this.current = tab;
            }
            ChatStatus.close();
            search.close();
        },
        isCurrent: function (tab) {
            return tab == this.current;
        }
    };

    var settings = {
        collapsed: true,
        lastStatus: ChatStatus.enums.online,
        sound: true,
        notification: true,
        layoutMin: false
    };

    var mergeSettings = function (values) {
        var self = Sidebar;
        for (var attrname in values) {
            self.settings[attrname] = values[attrname];
        }
        return self.settings;
    };

    // Behaviour events
    var toggleSidebar = function () {
        var self = Sidebar;
        self.settings.collapsed = !self.settings.collapsed;
        window.inGaiaChat_API.emit('chatVisibilityChange', {collapsed: self.settings.collapsed});
    };

    var openSidebar = function () {
        var self = Sidebar;
        self.settings.collapsed = false;
        window.inGaiaChat_API.emit('chatVisibilityChange', {collapsed: self.settings.collapsed});
    };

    var closeSidebar = function () {
        var self = Sidebar;
        self.settings.collapsed = true;
        window.inGaiaChat_API.emit('chatVisibilityChange', {collapsed: self.settings.collapsed});
    };

    window.inGaiaChat_API.on('chatToggle', toggleSidebar);
    window.inGaiaChat_API.on('chatOpen', openSidebar);
    window.inGaiaChat_API.on('chatClose', closeSidebar);

    var Sidebar = {
        errors: function () {
            return Errors;
        },
        search: search,
        settings: settings,
        tab: tab,
        toggle: toggleSidebar,
        open: openSidebar,
        close: closeSidebar,
        mergeSettings: mergeSettings
    };

    return Sidebar;
};

chatSidebarFactory.$inject = ['$rootScope', '$timeout', 'ChatAPI', 'ChatUser', 'ChatStatus', 'ChatMessageLog', 'ChatErrors'];

var factoriesModules = require('../factories.module');
factoriesModules.factory("ChatSidebar", chatSidebarFactory);
