var factoriesModules = require('../factories.module');

factoriesModules.factory("ChatStatus", ['ChatAPI', 'ChatUser', 'ChatMessageLog', function (ChatAPI, ChatUser, ChatMessageLog) {   
    var Status = {
        enums: {
            online: 1,
            busy: 2, 
            offline: 3,
            away: 4
        },
        current: null,
        isOpen: false,
        init: function (statusId) {
            var self = this;
            self.list = [
                {
                    id: self.enums.online,
                    name: ChatMessageLog.STATUS.ONLINE,
                    changedBy: "user"
                },
                {
                    id: self.enums.busy,
                    name: ChatMessageLog.STATUS.BUSY,
                    changedBy: "user"
                },
                {
                    id: self.enums.offline,
                    name: ChatMessageLog.STATUS.OFFLINE,
                    changedBy: "user"
                },
                {
                    id: self.enums.away,
                    name: ChatMessageLog.STATUS.AWAY,
                    changedBy: "user"
                }
            ];
            this.set(statusId) || (this.current = this.list[0]);
            // this.inactivityTimer();
            return this;
        },
        get: function (id) {
            var status = {};
            for (var i = 0; i < this.list.length; i++) {
                if (this.list[i].id == id) {
                    status = this.list[i];
                    break;
                }
            }
            return status;
        },
        set: function (statusId, changedBy) {
            if (!statusId) return;

            var self = this;
            changedBy = changedBy || "user";

            try {
                ChatAPI.proxy().server.changeStatus(statusId).then(function () {
                    self.current = self.get(statusId);
                    ChatUser.status = statusId;
                    if (changedBy) {
                        self.current.changedBy = changedBy;
                        // Only notify settings watch if user changed the status
                        if (changedBy == 'user')
                            window.inGaiaChat_API.emit('chatStatusChange', statusId);
                    }
                }, function (err) {
                    window.inGaiaChat_API.emit('changeStatusError');
                    console.error('Erro ao alterar status');
                });
            } catch(err) {
                window.inGaiaChat_API.emit('changeStatusError');
                console.error('Erro ao alterar status');
            };

            self.close();
        },
        open: function () {
            this.isOpen = true;
        },
        close: function () {
            this.isOpen = false;
        },
        toggle: function () {
            this.isOpen = !this.isOpen;
        },
        inactivityTimer: function () {
            var self = this;
            var timeoutID, inactivityTimeOut;
            inactivityTimeOut = (60 * 1000) * 30; // 30 minutes

            var events = ['mousedown', 'keypress', 'touchmove', 'MSPointerMove'/*, 'mousemove', 'DOMMouseScroll', 'mousewheel'*/];

            var setup = function () {
                events.forEach(function (event) {
                    window.addEventListener(event, resetTimer, false);
                })
                startTimer();
            };

            var startTimer = function () {
                timeoutID = window.setTimeout(goInactive, inactivityTimeOut);
            };

            var resetTimer = function () {
                window.clearTimeout(timeoutID);
                goActive();
            };

            var goInactive = function () {
                if (self.current.id == self.enums.online)
                    self.set(self.enums.away, "system"); // Set status away
            };

            var goActive = function () {
                if (self.current.id == self.enums.away && self.current.changedBy == "system")
                    self.set(self.enums.online, "system"); // Set status active
                startTimer();
            };

            setup();
        }
    };

    return Status;
}]);
