var factoriesModules = require('../factories.module');

factoriesModules.factory('ChatUtils', [function () {
    var getNetworkIds = function (networks) {
        if (!networks || !networks.length) {
            return null;
        } else {
            var result = [];
            networks.forEach(function (network) {
                if (network.type == "network") {
                    result.push(network.id);
                }
            });
            return result;
        }
    };

    // Returns a function, that, as long as it continues to be invoked, will not
    // be triggered. The function will be called after it stops being called for
    // N milliseconds. If `immediate` is passed, trigger the function on the
    // leading edge, instead of the trailing.
    var debounce = function debounce(func, wait, immediate) {
        var timeout;
        return function () {
            var context = this, args = arguments;
            var later = function () {
                timeout = null;
                if (!immediate) func.apply(context, args);
            };
            var callNow = immediate && !timeout;
            clearTimeout(timeout);
            timeout = setTimeout(later, wait);
            if (callNow) func.apply(context, args);
        };
    };

    /**
     * Merge isBlocked properties between contacts/blocked lists.
     * @method
     * @params {id: int, productTag: string}
     * @return {Array}
     */
    var mergeBlockeds = function (contacts, blockeds) {
        contacts.forEach(function(contact){
            contact.isBlocked = false;
        });

        blockeds.forEach(function(blocked){
            contacts.forEach(function(contact){
                if(blocked.id == contact.id) {
                    contact.isBlocked = true;
                }
            });       
        });

        return contacts;
    };

    /**
     * Return second array param without same objects from first array param
     * @method
     * @params {arr1: Array, arr2: Array, trackBy: String}
     * @return {Array}
     */
    var filterList = function (arr1, arr2, prop) {
        return arr1.filter(function (a) {
            return !arr2.some(function (b) {
                if (prop != undefined) {
                    return a[prop] == b[prop];
                } else {
                    return a == b;
                }
            });
        });
    };

    return {
        getNetworkIds: getNetworkIds,
        debounce: debounce,
        mergeBlockeds: mergeBlockeds,
        filterList: filterList
    };

}]);
